@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 # @Author  : llc
 # @Time    : 2022/4/1 16:54
+import re
 from copy import deepcopy
 from functools import wraps
 from typing import Optional, List, Dict, Any, Type, Callable, Tuple
@@ -12,8 +13,9 @@ from pydantic import BaseModel
 from .do_wrapper import _do_wrapper
 from .http import HTTPMethod
 from .models import Tag, Components
-from .utils import get_openapi_path, get_operation, get_responses, parse_and_store_tags, parse_parameters, \
-    validate_responses_type, parse_method, get_operation_id_for_path
+from .types import OpenAPIResponsesType
+from .utils import get_operation, get_responses, parse_and_store_tags, parse_parameters, validate_responses_type, \
+    parse_method, get_operation_id_for_path
 
 
 class APIBlueprint(Blueprint):
@@ -24,7 +26,7 @@ class APIBlueprint(Blueprint):
             *,
             abp_tags: Optional[List[Tag]] = None,
             abp_security: Optional[List[Dict[str, List[str]]]] = None,
-            abp_responses: Optional[Dict[str, Type[BaseModel]]] = None,
+            abp_responses: OpenAPIResponsesType = None,
             doc_ui: bool = True,
             **kwargs: Any
     ) -> None:
@@ -53,6 +55,28 @@ class APIBlueprint(Blueprint):
         self.abp_responses = abp_responses or {}
         self.doc_ui = doc_ui
 
+    def register_api(self, api: "APIBlueprint") -> None:
+        """Register a nested APIBlueprint"""
+        if api is self:
+            raise ValueError("Cannot register a api blueprint on itself")
+
+        for tag in api.tags:
+            if tag.name not in self.tag_names:
+                self.tags.append(tag)
+
+        for path_url, path_item in api.paths.items():
+            trail_slash = path_url.endswith("/")
+            # merge url_prefix and new api blueprint path url
+            uri = self.url_prefix.rstrip("/") + "/" + path_url.lstrip("/") if self.url_prefix else path_url
+            # strip the right slash
+            if not trail_slash:
+                uri = uri.rstrip("/")
+            self.paths[uri] = path_item
+
+        self.components_schemas.update(**api.components_schemas)
+
+        self.register_blueprint(api)
+
     def _do_decorator(
             self,
             rule: str,
@@ -124,12 +148,13 @@ class APIBlueprint(Blueprint):
                 parse_parameters(func, components_schemas=self.components_schemas, operation=operation)
             # parse response
             get_responses(combine_responses, extra_responses, self.components_schemas, operation)
-            uri = get_openapi_path(rule)
-            trail_slash = uri.endswith('/')
+            # /pet/<petId> --> /pet/{petId}
+            uri = re.sub(r"<([^<:]+:)?", "{", rule).replace(">", "}")
+            trail_slash = uri.endswith("/")
             # merge url_prefix and uri
             uri = self.url_prefix.rstrip("/") + "/" + uri.lstrip("/") if self.url_prefix else uri
             if not trail_slash:
-                uri = uri.rstrip('/')
+                uri = uri.rstrip("/")
             # parse method
             parse_method(uri, method, self.paths, operation)
             return header, cookie, path, query, form, body, combine_responses
@@ -145,14 +170,15 @@ class APIBlueprint(Blueprint):
             tags: Optional[List[Tag]] = None,
             summary: Optional[str] = None,
             description: Optional[str] = None,
-            responses: Optional[Dict[str, Type[BaseModel]]] = None,
+            responses: OpenAPIResponsesType = None,
             extra_responses: Optional[Dict[str, dict]] = None,
             security: Optional[List[Dict[str, List[Any]]]] = None,
             deprecated: Optional[bool] = None,
             operation_id: Optional[str] = None,
-            doc_ui: bool = True
+            doc_ui: bool = True,
+            **options: Any
     ) -> Callable:
-        """Decorator for rest api, like: app.route(methods=['GET'])"""
+        """Decorator for rest api, like: app.route(methods=["GET"])"""
 
         def decorator(func) -> Callable:
             header, cookie, path, query, form, body, combine_responses = \
@@ -186,7 +212,7 @@ class APIBlueprint(Blueprint):
                 )
                 return resp
 
-            options = {"methods": [HTTPMethod.GET]}
+            options.update({"methods": [HTTPMethod.GET]})
             self.add_url_rule(rule, view_func=wrapper, **options)
 
             return wrapper
@@ -200,14 +226,15 @@ class APIBlueprint(Blueprint):
             tags: Optional[List[Tag]] = None,
             summary: Optional[str] = None,
             description: Optional[str] = None,
-            responses: Optional[Dict[str, Type[BaseModel]]] = None,
+            responses: OpenAPIResponsesType = None,
             extra_responses: Optional[Dict[str, dict]] = None,
             security: Optional[List[Dict[str, List[Any]]]] = None,
             deprecated: Optional[bool] = None,
             operation_id: Optional[str] = None,
-            doc_ui: bool = True
+            doc_ui: bool = True,
+            **options: Any
     ) -> Callable:
-        """Decorator for rest api, like: app.route(methods=['POST'])"""
+        """Decorator for rest api, like: app.route(methods=["POST"])"""
 
         def decorator(func) -> Callable:
             header, cookie, path, query, form, body, combine_responses = \
@@ -241,7 +268,7 @@ class APIBlueprint(Blueprint):
                 )
                 return resp
 
-            options = {"methods": [HTTPMethod.POST]}
+            options.update({"methods": [HTTPMethod.POST]})
             self.add_url_rule(rule, view_func=wrapper, **options)
 
             return wrapper
@@ -255,14 +282,15 @@ class APIBlueprint(Blueprint):
             tags: Optional[List[Tag]] = None,
             summary: Optional[str] = None,
             description: Optional[str] = None,
-            responses: Optional[Dict[str, Type[BaseModel]]] = None,
+            responses: OpenAPIResponsesType = None,
             extra_responses: Optional[Dict[str, dict]] = None,
             security: Optional[List[Dict[str, List[Any]]]] = None,
             deprecated: Optional[bool] = None,
             operation_id: Optional[str] = None,
-            doc_ui: bool = True
+            doc_ui: bool = True,
+            **options: Any
     ) -> Callable:
-        """Decorator for rest api, like: app.route(methods=['PUT'])"""
+        """Decorator for rest api, like: app.route(methods=["PUT"])"""
 
         def decorator(func) -> Callable:
             header, cookie, path, query, form, body, combine_responses = \
@@ -296,7 +324,7 @@ class APIBlueprint(Blueprint):
                 )
                 return resp
 
-            options = {"methods": [HTTPMethod.PUT]}
+            options.update({"methods": [HTTPMethod.PUT]})
             self.add_url_rule(rule, view_func=wrapper, **options)
 
             return wrapper
@@ -310,14 +338,15 @@ class APIBlueprint(Blueprint):
             tags: Optional[List[Tag]] = None,
             summary: Optional[str] = None,
             description: Optional[str] = None,
-            responses: Optional[Dict[str, Type[BaseModel]]] = None,
+            responses: OpenAPIResponsesType = None,
             extra_responses: Optional[Dict[str, dict]] = None,
             security: Optional[List[Dict[str, List[Any]]]] = None,
             deprecated: Optional[bool] = None,
             operation_id: Optional[str] = None,
-            doc_ui: bool = True
+            doc_ui: bool = True,
+            **options: Any
     ) -> Callable:
-        """Decorator for rest api, like: app.route(methods=['DELETE'])"""
+        """Decorator for rest api, like: app.route(methods=["DELETE"])"""
 
         def decorator(func) -> Callable:
             header, cookie, path, query, form, body, combine_responses = \
@@ -351,7 +380,7 @@ class APIBlueprint(Blueprint):
                 )
                 return resp
 
-            options = {"methods": [HTTPMethod.DELETE]}
+            options.update({"methods": [HTTPMethod.DELETE]})
             self.add_url_rule(rule, view_func=wrapper, **options)
 
             return wrapper
@@ -365,14 +394,15 @@ class APIBlueprint(Blueprint):
             tags: Optional[List[Tag]] = None,
             summary: Optional[str] = None,
             description: Optional[str] = None,
-            responses: Optional[Dict[str, Type[BaseModel]]] = None,
+            responses: OpenAPIResponsesType = None,
             extra_responses: Optional[Dict[str, dict]] = None,
             security: Optional[List[Dict[str, List[Any]]]] = None,
             deprecated: Optional[bool] = None,
             operation_id: Optional[str] = None,
-            doc_ui: bool = True
+            doc_ui: bool = True,
+            **options: Any
     ) -> Callable:
-        """Decorator for rest api, like: app.route(methods=['PATCH'])"""
+        """Decorator for rest api, like: app.route(methods=["PATCH"])"""
 
         def decorator(func) -> Callable:
             header, cookie, path, query, form, body, combine_responses = \
@@ -406,7 +436,7 @@ class APIBlueprint(Blueprint):
                 )
                 return resp
 
-            options = {"methods": [HTTPMethod.PATCH]}
+            options.update({"methods": [HTTPMethod.PATCH]})
             self.add_url_rule(rule, view_func=wrapper, **options)
 
             return wrapper
