@@ -3,25 +3,26 @@
 # @Time    : 2021/4/30 14:25
 import json
 import os
+import re
 from copy import deepcopy
 from functools import wraps
-from io import StringIO
 from typing import Optional, List, Dict, Union, Any, Type, Callable, Tuple
 
-from flask import Flask, Blueprint, render_template, make_response
+from flask import Flask, Blueprint, render_template
 from flask.wrappers import Response
 from pydantic import BaseModel
 
 from .api_blueprint import APIBlueprint
+from .commands import openapi_command
 from .do_wrapper import _do_wrapper
 from .http import HTTPMethod
-from .markdown import openapi_to_markdown
 from .models import Info, APISpec, Tag, Components, Server
 from .models.common import Reference, ExternalDocumentation
 from .models.oauth import OAuthConfig
 from .models.security import SecurityScheme
-from .utils import get_openapi_path, get_operation, get_responses, parse_and_store_tags, parse_parameters, \
-    validate_responses_type, parse_method, get_operation_id_for_path
+from .types import OpenAPIResponsesType
+from .utils import get_operation, get_responses, parse_and_store_tags, parse_parameters, validate_responses_type, \
+    parse_method, get_operation_id_for_path
 
 
 class OpenAPI(Flask):
@@ -32,7 +33,7 @@ class OpenAPI(Flask):
             info: Optional[Info] = None,
             security_schemes: Optional[Dict[str, Union[SecurityScheme, Reference]]] = None,
             oauth_config: Optional[OAuthConfig] = None,
-            responses: Optional[Dict[str, Type[BaseModel]]] = None,
+            responses: OpenAPIResponsesType = None,
             doc_ui: bool = True,
             doc_expansion: str = "list",
             doc_prefix: str = "/openapi",
@@ -41,6 +42,7 @@ class OpenAPI(Flask):
             redoc_url: str = "/redoc",
             rapidoc_url: str = "/rapidoc",
             servers: Optional[List[Server]] = None,
+            external_docs: Optional[ExternalDocumentation] = None,
             **kwargs: Any
     ) -> None:
         """
@@ -56,22 +58,24 @@ class OpenAPI(Flask):
             doc_ui: add openapi document UI(swagger and redoc). Defaults to True.
             doc_expansion: String=["list"*, "full", "none"].
                           Controls the default expansion setting for the operations and tags.
-                          It can be 'list' (expands only the tags),
-                         'full' (expands the tags and operations) or 'none' (expands nothing).
+                          It can be "list" (expands only the tags),
+                         "full" (expands the tags and operations) or "none" (expands nothing).
                          see https://github.com/swagger-api/swagger-ui/blob/master/docs/usage/configuration.md
-            doc_prefix: URL prefix used for OpenAPI document and UI. Defaults to '/openapi'.
-            api_doc_url: The OpenAPI Spec documentation. Defaults to '/openapi.json'.
+            doc_prefix: URL prefix used for OpenAPI document and UI. Defaults to "/openapi".
+            api_doc_url: The OpenAPI Spec documentation. Defaults to "/openapi.json".
             swagger_url: The Swagger UI documentation. Defaults to `/swagger`.
             redoc_url: The Redoc UI documentation. Defaults to `/redoc`.
             rapidoc_url: The RapiDoc UI documentation. Defaults to `/rapidoc`.
             servers: An array of Server Objects, which provide connectivity information to a target server.
+            external_docs: Allows referencing an external resource for extended documentation.
+                           See: https://spec.openapis.org/oas/v3.0.3#external-documentation-object
             kwargs: Flask kwargs
         """
         super(OpenAPI, self).__init__(import_name, **kwargs)
 
         self.openapi_version = "3.0.3"
         if info is None:
-            info = Info(title='OpenAPI', version='1.0.0')
+            info = Info(title="OpenAPI", version="1.0.0")
         assert isinstance(info, Info), f"Info is required (got type {type(info)})"
         self.info = info
         self.security_schemes = security_schemes
@@ -94,17 +98,20 @@ class OpenAPI(Flask):
             self.init_doc()
         self.doc_expansion = doc_expansion
         self.severs = servers
+        self.external_docs = external_docs
+        # add openapi command
+        self.cli.add_command(openapi_command)
 
     def init_doc(self) -> None:
         """
         Provide Swagger UI, Redoc and Rapidoc
         """
         _here = os.path.dirname(__file__)
-        template_folder = os.path.join(_here, 'templates')
-        static_folder = os.path.join(template_folder, 'static')
+        template_folder = os.path.join(_here, "templates")
+        static_folder = os.path.join(template_folder, "static")
 
         blueprint = Blueprint(
-            'openapi',
+            "openapi",
             __name__,
             url_prefix=self.doc_prefix,
             template_folder=template_folder,
@@ -112,63 +119,47 @@ class OpenAPI(Flask):
         )
         blueprint.add_url_rule(
             rule=self.api_doc_url,
-            endpoint='api_doc',
+            endpoint="api_doc",
             view_func=lambda: self.api_doc
         )
         blueprint.add_url_rule(
             rule=self.swagger_url,
-            endpoint='swagger',
+            endpoint="swagger",
             view_func=lambda: render_template(
                 "swagger.html",
-                api_doc_url=self.api_doc_url.lstrip('/'),
+                api_doc_url=self.api_doc_url.lstrip("/"),
                 doc_expansion=self.doc_expansion,
                 oauth_config=self.oauth_config.dict() if self.oauth_config else None
             )
         )
         blueprint.add_url_rule(
             rule=self.redoc_url,
-            endpoint='redoc',
+            endpoint="redoc",
             view_func=lambda: render_template(
                 "redoc.html",
-                api_doc_url=self.api_doc_url.lstrip('/')
+                api_doc_url=self.api_doc_url.lstrip("/")
             )
         )
         blueprint.add_url_rule(
             rule=self.rapidoc_url,
-            endpoint='rapidoc',
+            endpoint="rapidoc",
             view_func=lambda: render_template(
                 "rapidoc.html",
-                api_doc_url=self.api_doc_url.lstrip('/')
+                api_doc_url=self.api_doc_url.lstrip("/")
             )
         )
         blueprint.add_url_rule(
-            rule='/markdown',
-            endpoint='markdown',
-            view_func=lambda: self.export_to_markdown()
-        )
-        blueprint.add_url_rule(
-            rule='/',
-            endpoint='index',
+            rule="/",
+            endpoint="index",
             view_func=lambda: render_template(
                 "index.html",
-                swagger_url=self.swagger_url.lstrip('/'),
-                redoc_url=self.redoc_url.lstrip('/'),
-                rapidoc_url=self.rapidoc_url.lstrip('/')
+                swagger_url=self.swagger_url.lstrip("/"),
+                redoc_url=self.redoc_url.lstrip("/"),
+                rapidoc_url=self.rapidoc_url.lstrip("/")
             )
         )
         self.register_blueprint(blueprint)
 
-    def export_to_markdown(self) -> Response:
-        """Export to markdown(Experimental)"""
-        md = StringIO()
-
-        md.write(openapi_to_markdown(self.api_doc))
-
-        r = make_response(md.getvalue())
-        r.headers['Content-Disposition'] = 'attachment; filename=api.md'
-
-        return r
-
     @property
     def api_doc(self) -> Dict:
         """Generate Specification json"""
@@ -176,10 +167,7 @@ class OpenAPI(Flask):
             openapi=self.openapi_version,
             info=self.info,
             servers=self.severs,
-            externalDocs=ExternalDocumentation(
-                url=f'{self.doc_prefix}/markdown',
-                description='Export to markdown'
-            )
+            externalDocs=self.external_docs
         )
         spec.tags = self.tags or None
         spec.paths = self.paths
@@ -265,7 +253,8 @@ class OpenAPI(Flask):
                 parse_parameters(func, components_schemas=self.components_schemas, operation=operation)
             # parse response
             get_responses(combine_responses, extra_responses, self.components_schemas, operation)
-            uri = get_openapi_path(rule)
+            # /pet/<petId> --> /pet/{petId}
+            uri = re.sub(r"<([^<:]+:)?", "{", rule).replace(">", "}")
             # parse method
             parse_method(uri, method, self.paths, operation)
             return header, cookie, path, query, form, body, combine_responses
@@ -281,14 +270,15 @@ class OpenAPI(Flask):
             tags: Optional[List[Tag]] = None,
             summary: Optional[str] = None,
             description: Optional[str] = None,
-            responses: Optional[Dict[str, Type[BaseModel]]] = None,
+            responses: OpenAPIResponsesType = None,
             extra_responses: Optional[Dict[str, dict]] = None,
             security: Optional[List[Dict[str, List[Any]]]] = None,
             deprecated: Optional[bool] = None,
             operation_id: Optional[str] = None,
-            doc_ui: bool = True
+            doc_ui: bool = True,
+            **options: Any
     ) -> Callable:
-        """Decorator for rest api, like: app.route(methods=['GET'])"""
+        """Decorator for rest api, like: app.route(methods=["GET"])"""
 
         def decorator(func) -> Callable:
             header, cookie, path, query, form, body, combine_responses = \
@@ -322,7 +312,7 @@ class OpenAPI(Flask):
                 )
                 return resp
 
-            options = {"methods": [HTTPMethod.GET]}
+            options.update({"methods": [HTTPMethod.GET]})
             self.add_url_rule(rule, view_func=wrapper, **options)
 
             return wrapper
@@ -336,14 +326,15 @@ class OpenAPI(Flask):
             tags: Optional[List[Tag]] = None,
             summary: Optional[str] = None,
             description: Optional[str] = None,
-            responses: Optional[Dict[str, Type[BaseModel]]] = None,
+            responses: OpenAPIResponsesType = None,
             extra_responses: Optional[Dict[str, dict]] = None,
             security: Optional[List[Dict[str, List[Any]]]] = None,
             deprecated: Optional[bool] = None,
             operation_id: Optional[str] = None,
-            doc_ui: bool = True
+            doc_ui: bool = True,
+            **options: Any
     ) -> Callable:
-        """Decorator for rest api, like: app.route(methods=['POST'])"""
+        """Decorator for rest api, like: app.route(methods=["POST"])"""
 
         def decorator(func) -> Callable:
             header, cookie, path, query, form, body, combine_responses = \
@@ -377,7 +368,7 @@ class OpenAPI(Flask):
                 )
                 return resp
 
-            options = {"methods": [HTTPMethod.POST]}
+            options.update({"methods": [HTTPMethod.POST]})
             self.add_url_rule(rule, view_func=wrapper, **options)
 
             return wrapper
@@ -391,14 +382,15 @@ class OpenAPI(Flask):
             tags: Optional[List[Tag]] = None,
             summary: Optional[str] = None,
             description: Optional[str] = None,
-            responses: Optional[Dict[str, Type[BaseModel]]] = None,
+            responses: OpenAPIResponsesType = None,
             extra_responses: Optional[Dict[str, dict]] = None,
             security: Optional[List[Dict[str, List[Any]]]] = None,
             deprecated: Optional[bool] = None,
             operation_id: Optional[str] = None,
-            doc_ui: bool = True
+            doc_ui: bool = True,
+            **options: Any
     ) -> Callable:
-        """Decorator for rest api, like: app.route(methods=['PUT'])"""
+        """Decorator for rest api, like: app.route(methods=["PUT"])"""
 
         def decorator(func) -> Callable:
             header, cookie, path, query, form, body, combine_responses = \
@@ -432,7 +424,7 @@ class OpenAPI(Flask):
                 )
                 return resp
 
-            options = {"methods": [HTTPMethod.PUT]}
+            options.update({"methods": [HTTPMethod.PUT]})
             self.add_url_rule(rule, view_func=wrapper, **options)
 
             return wrapper
@@ -446,14 +438,15 @@ class OpenAPI(Flask):
             tags: Optional[List[Tag]] = None,
             summary: Optional[str] = None,
             description: Optional[str] = None,
-            responses: Optional[Dict[str, Type[BaseModel]]] = None,
+            responses: OpenAPIResponsesType = None,
             extra_responses: Optional[Dict[str, dict]] = None,
             security: Optional[List[Dict[str, List[Any]]]] = None,
             deprecated: Optional[bool] = None,
             operation_id: Optional[str] = None,
-            doc_ui: bool = True
+            doc_ui: bool = True,
+            **options: Any
     ) -> Callable:
-        """Decorator for rest api, like: app.route(methods=['DELETE'])"""
+        """Decorator for rest api, like: app.route(methods=["DELETE"])"""
 
         def decorator(func) -> Callable:
             header, cookie, path, query, form, body, combine_responses = \
@@ -487,7 +480,7 @@ class OpenAPI(Flask):
                 )
                 return resp
 
-            options = {"methods": [HTTPMethod.DELETE]}
+            options.update({"methods": [HTTPMethod.DELETE]})
             self.add_url_rule(rule, view_func=wrapper, **options)
 
             return wrapper
@@ -501,14 +494,15 @@ class OpenAPI(Flask):
             tags: Optional[List[Tag]] = None,
             summary: Optional[str] = None,
             description: Optional[str] = None,
-            responses: Optional[Dict[str, Type[BaseModel]]] = None,
+            responses: OpenAPIResponsesType = None,
             extra_responses: Optional[Dict[str, dict]] = None,
             security: Optional[List[Dict[str, List[Any]]]] = None,
             deprecated: Optional[bool] = None,
             operation_id: Optional[str] = None,
-            doc_ui: bool = True
+            doc_ui: bool = True,
+            **options: Any
     ) -> Callable:
-        """Decorator for rest api, like: app.route(methods=['PATCH'])"""
+        """Decorator for rest api, like: app.route(methods=["PATCH"])"""
 
         def decorator(func) -> Callable:
             header, cookie, path, query, form, body, combine_responses = \
@@ -542,7 +536,7 @@ class OpenAPI(Flask):
                 )
                 return resp
 
-            options = {"methods": [HTTPMethod.PATCH]}
+            options.update({"methods": [HTTPMethod.PATCH]})
             self.add_url_rule(rule, view_func=wrapper, **options)
 
             return wrapper
