@@ -8,9 +8,8 @@ from http import HTTPStatus
 from typing import Dict, Type, Callable, List, Tuple, Any, ForwardRef
 
 import pydantic.typing
-from flask import Response as _Response
+from flask import Response as _Response, current_app
 from pydantic import BaseModel
-from werkzeug.routing import parse_rule
 
 from .http import HTTP_STATUS, HTTPMethod
 from .models import OPENAPI3_REF_TEMPLATE, OPENAPI3_REF_PREFIX, Tag
@@ -20,29 +19,16 @@ from .models.path import Operation, RequestBody, PathItem
 from .models.validation_error import UnprocessableEntity
 
 
-def get_openapi_path(rule: str) -> str:
-    """Flask route conversion to openapi path:
-    /pet/<petId> --> /pet/{petId}
-    """
-    uri = ''
-    for converter, args, variable in parse_rule(str(rule)):
-        if converter is None:
-            uri += variable
-            continue
-        uri += "{%s}" % variable
-    return uri
-
-
 def get_operation(func: Callable, *, summary: str = None, description: str = None) -> Operation:
     """Return an Operation object with summary and description."""
-    doc = inspect.getdoc(func) or ''
+    doc = inspect.getdoc(func) or ""
     doc = doc.strip()
-    lines = doc.split('\n')
+    lines = doc.split("\n")
     doc_summary = lines[0] or None
     if summary is None:
-        doc_description = lines[0] if len(lines) == 0 else '</br>'.join(lines[1:]) or None
+        doc_description = lines[0] if len(lines) == 0 else "</br>".join(lines[1:]) or None
     else:
-        doc_description = '</br>'.join(lines) or None
+        doc_description = "</br>".join(lines) or None
     operation = Operation(
         summary=summary or doc_summary,
         description=description or doc_description
@@ -53,12 +39,12 @@ def get_operation(func: Callable, *, summary: str = None, description: str = Non
 
 def get_operation_id_for_path(*, name: str, path: str, method: str) -> str:
     operation_id = name + path
-    operation_id = re.sub("[^0-9a-zA-Z_]", "_", operation_id)
+    operation_id = re.sub(r"\W", "_", operation_id)
     operation_id = operation_id + "_" + method.lower()
     return operation_id
 
 
-def get_func_parameter(func: Callable, func_globals: Dict[str, Any], *, parameter_name='path') -> Type[BaseModel]:
+def get_func_parameter(func: Callable, *, parameter_name="path") -> Type[BaseModel]:
     """Get view-func parameters.
     parameter_name has six parameters to choose from: path, query, form, body, header, cookie.
     """
@@ -67,8 +53,12 @@ def get_func_parameter(func: Callable, func_globals: Dict[str, Any], *, paramete
     annotation = param.annotation if param else None
     if isinstance(annotation, str):
         # PEP563
+        while hasattr(func, "__wrapped__"):
+            # Find globalns for the unwrapped func.
+            func = func.__wrapped__
+        globalns = getattr(func, "__globals__", {})
         annotation = ForwardRef(annotation)
-        annotation = pydantic.typing.evaluate_forwardref(annotation, func_globals, func_globals)
+        annotation = pydantic.typing.evaluate_forwardref(annotation, globalns, globalns)
     return annotation
 
 
@@ -85,22 +75,21 @@ def parse_header(header: Type[BaseModel]) -> Tuple[List[Parameter], dict]:
     schema = get_schema(header)
     parameters = []
     components_schemas = dict()
-    properties = schema.get('properties')
-    definitions = schema.get('definitions')
+    properties = schema.get("properties", {})
+    definitions = schema.get("definitions", {})
 
-    if properties:
-        for name, value in properties.items():
-            data = {
-                "name": name,
-                "in": ParameterInType.header,
-                "description": value.get("description"),
-                "required": name in schema.get("required", []),
-                "schema": Schema(**value)
-            }
-            parameters.append(Parameter(**data))
-    if definitions:
-        for name, value in definitions.items():
-            components_schemas[name] = Schema(**value)
+    for name, value in properties.items():
+        data = {
+            "name": name,
+            "in": ParameterInType.header,
+            "description": value.get("description"),
+            "required": name in schema.get("required", []),
+            "schema": Schema(**value)
+        }
+        parameters.append(Parameter(**data))
+
+    for name, value in definitions.items():
+        components_schemas[name] = Schema(**value)
 
     return parameters, components_schemas
 
@@ -110,22 +99,21 @@ def parse_cookie(cookie: Type[BaseModel]) -> Tuple[List[Parameter], dict]:
     schema = get_schema(cookie)
     parameters = []
     components_schemas = dict()
-    properties = schema.get('properties')
-    definitions = schema.get('definitions')
+    properties = schema.get("properties", {})
+    definitions = schema.get("definitions", {})
 
-    if properties:
-        for name, value in properties.items():
-            data = {
-                "name": name,
-                "in": ParameterInType.cookie,
-                "description": value.get("description"),
-                "required": name in schema.get("required", []),
-                "schema": Schema(**value)
-            }
-            parameters.append(Parameter(**data))
-    if definitions:
-        for name, value in definitions.items():
-            components_schemas[name] = Schema(**value)
+    for name, value in properties.items():
+        data = {
+            "name": name,
+            "in": ParameterInType.cookie,
+            "description": value.get("description"),
+            "required": name in schema.get("required", []),
+            "schema": Schema(**value)
+        }
+        parameters.append(Parameter(**data))
+
+    for name, value in definitions.items():
+        components_schemas[name] = Schema(**value)
 
     return parameters, components_schemas
 
@@ -135,22 +123,21 @@ def parse_path(path: Type[BaseModel]) -> Tuple[List[Parameter], dict]:
     schema = get_schema(path)
     parameters = []
     components_schemas = dict()
-    properties = schema.get('properties')
-    definitions = schema.get('definitions')
+    properties = schema.get("properties", {})
+    definitions = schema.get("definitions", {})
 
-    if properties:
-        for name, value in properties.items():
-            data = {
-                "name": name,
-                "in": ParameterInType.path,
-                "description": value.get("description"),
-                "required": True,
-                "schema": Schema(**value)
-            }
-            parameters.append(Parameter(**data))
-    if definitions:
-        for name, value in definitions.items():
-            components_schemas[name] = Schema(**value)
+    for name, value in properties.items():
+        data = {
+            "name": name,
+            "in": ParameterInType.path,
+            "description": value.get("description"),
+            "required": True,
+            "schema": Schema(**value)
+        }
+        parameters.append(Parameter(**data))
+
+    for name, value in definitions.items():
+        components_schemas[name] = Schema(**value)
 
     return parameters, components_schemas
 
@@ -160,22 +147,21 @@ def parse_query(query: Type[BaseModel]) -> Tuple[List[Parameter], dict]:
     schema = get_schema(query)
     parameters = []
     components_schemas = dict()
-    properties = schema.get('properties')
-    definitions = schema.get('definitions')
+    properties = schema.get("properties", {})
+    definitions = schema.get("definitions", {})
 
-    if properties:
-        for name, value in properties.items():
-            data = {
-                "name": name,
-                "in": ParameterInType.query,
-                "description": value.get("description"),
-                "required": name in schema.get("required", []),
-                "schema": Schema(**value)
-            }
-            parameters.append(Parameter(**data))
-    if definitions:
-        for name, value in definitions.items():
-            components_schemas[name] = Schema(**value)
+    for name, value in properties.items():
+        data = {
+            "name": name,
+            "in": ParameterInType.query,
+            "description": value.get("description"),
+            "required": name in schema.get("required", []),
+            "schema": Schema(**value)
+        }
+        parameters.append(Parameter(**data))
+
+    for name, value in definitions.items():
+        components_schemas[name] = Schema(**value)
 
     return parameters, components_schemas
 
@@ -183,36 +169,34 @@ def parse_query(query: Type[BaseModel]) -> Tuple[List[Parameter], dict]:
 def parse_form(form: Type[BaseModel]) -> Tuple[Dict[str, MediaType], dict]:
     """Parse form model"""
     schema = get_schema(form)
-    content = None
     components_schemas = dict()
-    properties = schema.get('properties')
-    definitions = schema.get('definitions')
+    properties = schema.get("properties", {})
+    definitions = schema.get("definitions", {})
 
-    if properties:
-        title = schema.get('title')
-        components_schemas[title] = Schema(**schema)
-        encoding = {}
-        for k, v in form.schema().get('properties', {}).items():
-            if v.get('type') == 'array':
-                encoding[k] = {'style': 'form'}
+    assert properties, f"{form.__name__}'s properties cannot be empty."
 
-        content = {
-            "multipart/form-data": MediaType(
-                **{
-                    "schema": Schema(
-                        **{
-                            "$ref": f"{OPENAPI3_REF_PREFIX}/{title}"
-                        }
-                    ),
-                    "encoding": encoding
-                }
-            )
-        }
-    if definitions:
-        for name, value in definitions.items():
-            components_schemas[name] = Schema(**value)
+    title = schema.get("title")
+    components_schemas[title] = Schema(**schema)
+    encoding = {}
+    for k, v in properties.items():
+        if v.get("type") == "array":
+            encoding[k] = {"style": "form"}
 
-    assert content is not None, f"{form.__name__}'s properties cannot be empty."
+    content = {
+        "multipart/form-data": MediaType(
+            **{
+                "schema": Schema(
+                    **{
+                        "$ref": f"{OPENAPI3_REF_PREFIX}/{title}"
+                    }
+                ),
+                "encoding": encoding
+            }
+        )
+    }
+
+    for name, value in definitions.items():
+        components_schemas[name] = Schema(**value)
 
     return content, components_schemas
 
@@ -220,39 +204,26 @@ def parse_form(form: Type[BaseModel]) -> Tuple[Dict[str, MediaType], dict]:
 def parse_body(body: Type[BaseModel]) -> Tuple[Dict[str, MediaType], dict]:
     """Parse body model"""
     schema = get_schema(body)
-    content = None
     components_schemas = dict()
-    properties = schema.get('properties')
-    definitions = schema.get('definitions')
+    definitions = schema.get("definitions", {})
 
-    if properties:
-        title = schema.get('title')
-        components_schemas[title] = Schema(**schema)
-        content = {
-            "application/json": MediaType(
-                **{
-                    "schema": Schema(
-                        **{
-                            "$ref": f"{OPENAPI3_REF_PREFIX}/{title}"
-                        }
-                    )
-                }
-            )
-        }
-    if definitions:
-        for name, value in definitions.items():
-            components_schemas[name] = Schema(**value)
+    title = schema.get("title")
+    components_schemas[title] = Schema(**schema)
 
-    if content is None:
-        content = {
-            "application/json": MediaType(
-                **{
-                    "schema": Schema(
-                        **schema
-                    )
-                }
-            )
-        }
+    content = {
+        "application/json": MediaType(
+            **{
+                "schema": Schema(
+                    **{
+                        "$ref": f"{OPENAPI3_REF_PREFIX}/{title}"
+                    }
+                )
+            }
+        )
+    }
+
+    for name, value in definitions.items():
+        components_schemas[name] = Schema(**value)
 
     return content, components_schemas
 
@@ -290,36 +261,40 @@ def get_responses(
             }
         )
         _schemas[UnprocessableEntity.__name__] = Schema(**UnprocessableEntity.schema())
-    # if not responses.get("500"):
-    #     _responses["500"] = Response(description=HTTP_STATUS["500"])
-    # handle extra_responses
     for key, response in responses.items():
-        assert inspect.isclass(response) and \
-               issubclass(response, BaseModel), f" {response} is invalid `pydantic.BaseModel`"
-        schema = response.schema(ref_template=OPENAPI3_REF_TEMPLATE)
-        _responses[key] = Response(
-            description=HTTP_STATUS.get(key, ""),
-            content={
-                "application/json": MediaType(
-                    **{
-                        "schema": Schema(
-                            **{
-                                "$ref": f"{OPENAPI3_REF_PREFIX}/{response.__name__}"
-                            }
-                        )
-                    }
-                )
-            }
-        )
-        _schemas[response.__name__] = Schema(**schema)
-        definitions = schema.get('definitions')
-        if definitions:
-            for name, value in definitions.items():
-                _schemas[name] = Schema(**value)
+        # Verify that the response is a class and that class is a subclass of `pydantic.BaseModel`
+        if inspect.isclass(response) and issubclass(response, BaseModel):
+            schema = response.schema(ref_template=OPENAPI3_REF_TEMPLATE)
+            _responses[key] = Response(
+                description=HTTP_STATUS.get(key, ""),
+                content={
+                    "application/json": MediaType(
+                        **{
+                            "schema": Schema(
+                                **{
+                                    "$ref": f"{OPENAPI3_REF_PREFIX}/{response.__name__}"
+                                }
+                            )
+                        }
+                    )
+                }
+            )
+            _schemas[response.__name__] = Schema(**schema)
+            definitions = schema.get("definitions")
+            if definitions:
+                for name, value in definitions.items():
+                    _schemas[name] = Schema(**value)
+        # Verify that if the response is None, because http status code "204" means return "No Content"
+        elif response is None:
+            _responses[key] = Response(
+                description=HTTP_STATUS.get(key, ""),
+            )
+        else:
+            raise TypeError(f"{response} is invalid `pydantic.BaseModel`.")
     # handle extra_responses
     for key, value in extra_responses.items():
         # key "200" value {"content":{"text/csv":{"schema":{"type": "string"}}}}
-        extra_content = value.get('content', {})
+        extra_content = value.get("content", {})
         if extra_content:
             # {"text/csv":{"schema":{"type": "string"}}}
             if _responses.get(key) and isinstance(extra_content, dict):
@@ -340,10 +315,12 @@ def validate_responses_type(responses: Dict[str, Any]) -> None:
 
 def validate_response(resp: Any, responses: Dict[str, Type[BaseModel]]) -> None:
     """Validate response"""
-    print("Warning: "
-          "You are using `VALIDATE_RESPONSE=True`, "
-          "please do not use it in the production environment, "
-          "because it will reduce the performance.")
+    if not current_app.config.get("FLASK_OPENAPI_DISABLE_WARNINGS", False):
+        print("Warning: "
+              "You are using `FLASK_OPENAPI_VALIDATE_RESPONSE=True`, "
+              "please do not use it in the production environment, "
+              "because it will reduce the performance. "
+              "Note, you can disable this warning with `Flask.config['FLASK_OPENAPI_DISABLE_WARNINGS'] = True`")
     if isinstance(resp, tuple):  # noqa
         _resp, status_code = resp[:2]
     elif isinstance(resp, _Response):
@@ -364,7 +341,11 @@ def validate_response(resp: Any, responses: Dict[str, Type[BaseModel]]) -> None:
     assert inspect.isclass(resp_model) and \
            issubclass(resp_model, BaseModel), f"{resp_model} is invalid `pydantic.BaseModel`"
     try:
-        resp_model(**_resp)
+        if resp_model.__custom_root_type__:
+            # https://pydantic-docs.helpmanual.io/usage/models/#custom-root-types
+            resp_model(__root__=_resp)
+        else:
+            resp_model(**_resp)
     except TypeError:
         raise TypeError(f"`{resp_model.__name__}` validation failed, must be a mapping.")
 
@@ -403,17 +384,13 @@ def parse_parameters(
     :param operation: `models.path.py` Operation
     :param doc_ui: add openapi document UI(swagger and redoc). Defaults to True.
     """
-    func_globals = getattr(func, '__globals__', {})
-    # extra globals used in decorator(wrapper) for func, wrapper.__extra_globals__ = getattr(func, '__globals__', {})
-    func_globals.update(**getattr(func, '__extra_globals__', {}))  # noqa
-
     parameters = []
-    header = get_func_parameter(func, func_globals, parameter_name='header')
-    cookie = get_func_parameter(func, func_globals, parameter_name='cookie')
-    path = get_func_parameter(func, func_globals, parameter_name='path')
-    query = get_func_parameter(func, func_globals, parameter_name='query')
-    form = get_func_parameter(func, func_globals, parameter_name='form')
-    body = get_func_parameter(func, func_globals, parameter_name='body')
+    header = get_func_parameter(func, parameter_name="header")
+    cookie = get_func_parameter(func, parameter_name="cookie")
+    path = get_func_parameter(func, parameter_name="path")
+    query = get_func_parameter(func, parameter_name="query")
+    form = get_func_parameter(func, parameter_name="form")
+    body = get_func_parameter(func, parameter_name="body")
     if doc_ui is False:
         return header, cookie, path, query, form, body
     if header:
